package com.seikoinstruments.sdk.thermalprinter.sample;

import java.util.Locale;

import com.seikoinstruments.sdk.thermalprinter.PrinterManager;
import com.seikoinstruments.sdk.thermalprinter.sample.R;

import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.preference.PreferenceFragment;
import android.preference.PreferenceManager;
import android.support.v4.app.FragmentActivity;
import android.widget.Toast;


/**
 * Activity of setting screen.
 */
public class SettingsActivity extends FragmentActivity {

    protected static final int SEND_TIMEOUT_MIN = 100;
    protected static final int SEND_TIMEOUT_MAX = 90000;
    protected static final int SEND_TIMEOUT_DEFAULT = 10000;
    protected static final int RESPONSE_TIMEOUT_MIN = 100;
    protected static final int RESPONSE_TIMEOUT_MAX = 90000;
    protected static final int RESPONSE_TIMEOUT_DEFAULT = 10000;
    protected static final int SOCKET_KEEPING_TIME_MIN = 60000;
    protected static final int SOCKET_KEEPING_TIME_MAX = 300000;
    protected static final int SOCKET_KEEPING_TIME_DEFAULT = 300000;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Display the fragment
        getFragmentManager().beginTransaction()
                .replace(android.R.id.content, new SettingPreferenceFragment())
                .commit();
    }

    public static class SettingPreferenceFragment extends PreferenceFragment {
        /** PrinterManager(SDK) */
        private PrinterManager mPrinterManager;

        /** change summary */
        private SharedPreferences.OnSharedPreferenceChangeListener mListener =
                new SharedPreferences.OnSharedPreferenceChangeListener() {

                    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {

                        if (getString(R.string.key_send_timeout).equals(key)) {
                            // send timeout
                            EditTextPreference txtSendTimeout
                                    = (EditTextPreference) getPreferenceScreen().findPreference(getString(R.string.key_send_timeout));
                            if (txtSendTimeout != null) {
                                String strSendTimeout = txtSendTimeout.getText();
                                int intSendTimeout = StringUtil.getInt(strSendTimeout);
                                int validSendTimeout = setSendTimeout(intSendTimeout);
                                if (intSendTimeout != validSendTimeout) {
                                    txtSendTimeout.setText(Integer.toString(validSendTimeout));
                                    Toast.makeText(getActivity(), R.string.invalid_value, Toast.LENGTH_SHORT).show();
                                } else {
                                    txtSendTimeout.setSummary(txtSendTimeout.getText() + getString(R.string.timeout_unit));
                                }
                            }

                        } else if (getString(R.string.key_receive_timeout).equals(key)) {
                            // receive timeout
                            EditTextPreference txtReceiveTimeout
                                    = (EditTextPreference) getPreferenceScreen().findPreference(getString(R.string.key_receive_timeout));
                            if (txtReceiveTimeout != null) {
                                String strReceiveTimeout = txtReceiveTimeout.getText();
                                int intReceiveTimeout = StringUtil.getInt(strReceiveTimeout);
                                int validReceiveTimeout = setReceiveTimeout(intReceiveTimeout);
                                if (intReceiveTimeout != validReceiveTimeout) {
                                    txtReceiveTimeout.setText(Integer.toString(validReceiveTimeout));
                                    Toast.makeText(getActivity(), R.string.invalid_value, Toast.LENGTH_SHORT).show();
                                } else {
                                    txtReceiveTimeout.setSummary(txtReceiveTimeout.getText() + getString(R.string.timeout_unit));
                                }
                            }

                        } else if (getString(R.string.key_socket_keeping_time).equals(key)) {
                            // socket keeping time
                            EditTextPreference txtSocketKeepingTime
                                    = (EditTextPreference) getPreferenceScreen().findPreference(getString(R.string.key_socket_keeping_time));
                            if (txtSocketKeepingTime != null) {
                                String strSocketKeepingTime = txtSocketKeepingTime.getText();
                                int intSocketKeepingTime = StringUtil.getInt(strSocketKeepingTime);
                                int validSocketKeepingTime = setSocketKeepingTime(intSocketKeepingTime);
                                if (intSocketKeepingTime != validSocketKeepingTime) {
                                    txtSocketKeepingTime.setText(Integer.toString(validSocketKeepingTime));
                                    Toast.makeText(getActivity(), R.string.invalid_value, Toast.LENGTH_SHORT).show();
                                } else {
                                    txtSocketKeepingTime.setSummary(txtSocketKeepingTime.getText() + getString(R.string.timeout_unit));
                                }
                            }

                        } else if (getString(R.string.key_international_character).equals(key)) {
                            // international character
                            ListPreference LstInternationalCharacter
                                    = (ListPreference) getPreferenceScreen().findPreference(getString(R.string.key_international_character));
                            if (LstInternationalCharacter != null) {
                                String strInternationalCharacter = LstInternationalCharacter.getValue();
                                int intInternationalCharacter = StringUtil.getInt(strInternationalCharacter);
                                int validInternationalCharacter = setInternationalCharacter(intInternationalCharacter);
                                if (intInternationalCharacter != validInternationalCharacter) {
                                    LstInternationalCharacter.setValue(Integer.toString(validInternationalCharacter));
                                    Toast.makeText(getActivity(), R.string.invalid_value, Toast.LENGTH_SHORT).show();
                                } else {
                                    LstInternationalCharacter.setSummary(LstInternationalCharacter.getEntry());
                                }
                            }

                        } else if (getString(R.string.key_code_page).equals(key)) {
                            // code page
                            ListPreference LstCodePage
                                    = (ListPreference) getPreferenceScreen().findPreference(getString(R.string.key_code_page));
                            if (LstCodePage != null) {
                                String strCodePage = LstCodePage.getValue();
                                int intCodePage = StringUtil.getInt(strCodePage);
                                int validCodePage = setCodePage(intCodePage);
                                if (intCodePage != validCodePage) {
                                    LstCodePage.setValue(Integer.toString(validCodePage));
                                    Toast.makeText(getActivity(), R.string.invalid_value, Toast.LENGTH_SHORT).show();
                                } else {
                                    LstCodePage.setSummary(LstCodePage.getEntry());
                                }
                            }
                        }

                    }
                };

        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            addPreferencesFromResource(R.xml.preference);

            SampleApplication application = (SampleApplication) getActivity().getApplication();
            mPrinterManager = application.getPrinterManager();

            final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(getActivity());
            Editor editor = pref.edit();

            // send timeout
            EditTextPreference txtSendTimeout
                    = (EditTextPreference) getPreferenceScreen().findPreference(getString(R.string.key_send_timeout));
            if (txtSendTimeout != null) {
                String sendTimeout = txtSendTimeout.getText();
                if (StringUtil.isEmpty(sendTimeout)) {
                    sendTimeout = getString(R.string.send_timeout_default);
                    editor.putString(getString(R.string.key_send_timeout), sendTimeout);
                } else {
                    String strSendTimeout = txtSendTimeout.getText();
                    int intSendTimeout = StringUtil.getInt(strSendTimeout);
                    int validSendTimeout = setSendTimeout(intSendTimeout);
                    if (intSendTimeout != validSendTimeout) {
                        sendTimeout = Integer.toString(validSendTimeout);
                        editor.putString(getString(R.string.key_send_timeout), sendTimeout);
                    }
                }
                txtSendTimeout.setSummary(sendTimeout + getString(R.string.timeout_unit));
            }

            // receive timeout
            EditTextPreference txtReceiveTimeout
                    = (EditTextPreference) getPreferenceScreen().findPreference(getString(R.string.key_receive_timeout));
            String receiveTimeout = txtReceiveTimeout.getText();
            if (StringUtil.isEmpty(receiveTimeout)) {
                receiveTimeout = getString(R.string.receive_timeout_default);
                editor.putString(getString(R.string.key_receive_timeout), receiveTimeout);
            } else {
                String strReceiveTimeout = txtReceiveTimeout.getText();
                int intReceiveTimeout = StringUtil.getInt(strReceiveTimeout);
                int validReceiveTimeout = setReceiveTimeout(intReceiveTimeout);
                if (intReceiveTimeout != validReceiveTimeout) {
                    receiveTimeout = Integer.toString(validReceiveTimeout);
                    editor.putString(getString(R.string.key_receive_timeout), receiveTimeout);
                }
            }
            txtReceiveTimeout.setSummary(receiveTimeout + getString(R.string.timeout_unit));

            // socket keeping time
            EditTextPreference txtSocketKeepingTime
                    = (EditTextPreference) getPreferenceScreen().findPreference(getString(R.string.key_socket_keeping_time));
            String socketKeepingTime = txtSocketKeepingTime.getText();
            if (StringUtil.isEmpty(socketKeepingTime)) {
                socketKeepingTime = getString(R.string.socket_keeping_time_default);
                editor.putString(getString(R.string.key_socket_keeping_time), socketKeepingTime);
            } else {
                String strSocketKeepingTime = txtSocketKeepingTime.getText();
                int intSocketKeepingTime = StringUtil.getInt(strSocketKeepingTime);
                int validSocketKeepingTime = setSocketKeepingTime(intSocketKeepingTime);
                if (intSocketKeepingTime != validSocketKeepingTime) {
                    socketKeepingTime = Integer.toString(validSocketKeepingTime);
                    editor.putString(getString(R.string.key_socket_keeping_time), socketKeepingTime);
                }
            }
            txtSocketKeepingTime.setSummary(socketKeepingTime + getString(R.string.timeout_unit));

            // international character
            ListPreference LstInternationalCharacter
                    = (ListPreference) getPreferenceScreen().findPreference(getString(R.string.key_international_character));
            CharSequence internationalCharacter = LstInternationalCharacter.getEntry();
            if (StringUtil.isEmpty(internationalCharacter)) {
                if (Locale.JAPAN.equals(Locale.getDefault())) {
                    internationalCharacter = getString(R.string.international_character_default_jp);
                    editor.putString(getString(R.string.key_international_character), getString(R.string.international_character_default_jp_values));
                } else {
                    internationalCharacter = getString(R.string.international_character_default_en);
                    editor.putString(getString(R.string.key_international_character), getString(R.string.international_character_default_en_values));
                }

            } else {
                String strInternationalCharacter = LstInternationalCharacter.getValue();
                int intInternationalCharacter = StringUtil.getInt(strInternationalCharacter);
                int validInternationalCharacter = setInternationalCharacter(intInternationalCharacter);
                if (intInternationalCharacter != validInternationalCharacter) {
                    internationalCharacter = getInternationalCharacterString(validInternationalCharacter);
                    editor.putString(getString(R.string.key_international_character), Integer.toString(validInternationalCharacter));
                    LstInternationalCharacter.setValue(Integer.toString(validInternationalCharacter));
                }
            }
            LstInternationalCharacter.setSummary(internationalCharacter);

            // code page
            ListPreference LstCodePage
                    = (ListPreference) getPreferenceScreen().findPreference(getString(R.string.key_code_page));
            CharSequence codePage = LstCodePage.getEntry();
            if (StringUtil.isEmpty(codePage)) {
                if (Locale.JAPAN.equals(Locale.getDefault())) {
                    codePage = getString(R.string.code_page_default_jp);
                    editor.putString(getString(R.string.key_code_page), getString(R.string.code_page_default_jp_values));
                } else {
                    codePage = getString(R.string.code_page_default_en);
                    editor.putString(getString(R.string.key_code_page), getString(R.string.code_page_default_en_values));
                }

            } else {
                String strCodePage = LstCodePage.getValue();
                int intCodePage = StringUtil.getInt(strCodePage);
                int validCodePage = setCodePage(intCodePage);
                if (intCodePage != validCodePage) {
                    codePage = getCodePageString(validCodePage);
                    editor.putString(getString(R.string.key_code_page), Integer.toString(validCodePage));
                    LstCodePage.setValue(Integer.toString(validCodePage));
                }
            }
            LstCodePage.setSummary(codePage);

            // sdk version
            EditTextPreference txtSdkVersion
                    = (EditTextPreference) getPreferenceScreen().findPreference(getString(R.string.key_sdk_version));
            String sdkVersion = mPrinterManager.getVersion() ;
            if (StringUtil.isEmpty(sdkVersion)) {
                sdkVersion = getString(R.string.sdk_version_default);
            }
            editor.putString(getString(R.string.key_sdk_version), sdkVersion);
            txtSdkVersion.setSelectable(false) ;
            txtSdkVersion.setSummary(sdkVersion);

            editor.commit();
        }

        @Override
        public void onResume() {
            super.onResume();
            getPreferenceScreen().getSharedPreferences().registerOnSharedPreferenceChangeListener(mListener);
        }


        @Override
        public void onPause() {
            super.onPause();
            getPreferenceScreen().getSharedPreferences().unregisterOnSharedPreferenceChangeListener(mListener);
        }


        private int setSendTimeout(int sendTimeout) {
            int time;
            if (mPrinterManager != null) {
                mPrinterManager.setSendTimeout(sendTimeout);
                time = mPrinterManager.getSendTimeout();

            } else {
                if (sendTimeout < SEND_TIMEOUT_MIN || sendTimeout > SEND_TIMEOUT_MAX) {
                    time = SEND_TIMEOUT_DEFAULT;
                } else {
                    time = sendTimeout;
                }
            }

            return time;
        }


        private int setReceiveTimeout(int receiveTimeout) {
            int time;
            if (mPrinterManager != null) {
                mPrinterManager.setReceiveTimeout(receiveTimeout);
                time = mPrinterManager.getReceiveTimeout();

            } else {
                if (receiveTimeout < RESPONSE_TIMEOUT_MIN || receiveTimeout > RESPONSE_TIMEOUT_MAX) {
                    time = RESPONSE_TIMEOUT_DEFAULT;
                } else {
                    time = receiveTimeout;
                }
            }

            return time;
        }


        private int setSocketKeepingTime(int socketKeepingTime) {
            int time;
            if (mPrinterManager != null) {
                mPrinterManager.setSocketKeepingTime(socketKeepingTime);
                time = mPrinterManager.getSocketKeepingTime();

            } else {
                if (socketKeepingTime < SOCKET_KEEPING_TIME_MIN || socketKeepingTime > SOCKET_KEEPING_TIME_MAX) {
                    time = SOCKET_KEEPING_TIME_DEFAULT;
                } else {
                    time = socketKeepingTime;
                }
            }

            return time;
        }


        private int setInternationalCharacter(int internationalCharacter) {
            int international;
            if (mPrinterManager != null) {
                mPrinterManager.setInternationalCharacter(internationalCharacter);
                international = mPrinterManager.getInternationalCharacter();

            } else {
                international = internationalCharacter;
            }

            return international;
        }


        private int setCodePage(int codePage) {
            int code;
            if (mPrinterManager != null) {
                mPrinterManager.setCodePage(codePage);
                code = mPrinterManager.getCodePage();

            } else {
                code = codePage;
            }

            return code;
        }


        private String getInternationalCharacterString(int code) {
            String internationalCharacter;
            String[] internationalCharacterList = getResources().getStringArray(R.array.international_character_list);
            switch (code) {
                case PrinterManager.COUNTRY_USA:
                    internationalCharacter = internationalCharacterList[0];
                    break;

                case PrinterManager.COUNTRY_FRANCE:
                    internationalCharacter = internationalCharacterList[1];
                    break;

                case PrinterManager.COUNTRY_GERMANY:
                    internationalCharacter = internationalCharacterList[2];
                    break;

                case PrinterManager.COUNTRY_ENGLAND:
                    internationalCharacter = internationalCharacterList[3];
                    break;

                case PrinterManager.COUNTRY_DENMARK_1:
                    internationalCharacter = internationalCharacterList[4];
                    break;

                case PrinterManager.COUNTRY_SWEDEN:
                    internationalCharacter = internationalCharacterList[5];
                    break;

                case PrinterManager.COUNTRY_ITALY:
                    internationalCharacter = internationalCharacterList[6];
                    break;

                case PrinterManager.COUNTRY_SPAIN:
                    internationalCharacter = internationalCharacterList[7];
                    break;

                case PrinterManager.COUNTRY_JAPAN:
                    internationalCharacter = internationalCharacterList[8];
                    break;

                case PrinterManager.COUNTRY_NORWAY:
                    internationalCharacter = internationalCharacterList[9];
                    break;

                case PrinterManager.COUNTRY_DENMARK_2:
                    internationalCharacter = internationalCharacterList[10];
                    break;

                case PrinterManager.COUNTRY_SPAIN_2:
                    internationalCharacter = internationalCharacterList[11];
                    break;

                case PrinterManager.COUNTRY_LATIN_AMERICA:
                    internationalCharacter = internationalCharacterList[12];
                    break;

                case PrinterManager.COUNTRY_ARABIA:
                    internationalCharacter = internationalCharacterList[13];
                    break;

                default:
                    internationalCharacter = "";
                    break;
            }

            return internationalCharacter;
        }


        private String getCodePageString(int code) {
            String codePage;
            String[] codePageList = getResources().getStringArray(R.array.code_page_list);
            switch (code) {
                case PrinterManager.CODE_PAGE_KATAKANA:
                    codePage = codePageList[0];
                    break;

                case PrinterManager.CODE_PAGE_1252:
                    codePage = codePageList[1];
                    break;

                case PrinterManager.CODE_PAGE_437:
                    codePage = codePageList[2];
                    break;

                case PrinterManager.CODE_PAGE_850:
                    codePage = codePageList[3];
                    break;

                case PrinterManager.CODE_PAGE_860:
                    codePage = codePageList[4];
                    break;

                case PrinterManager.CODE_PAGE_863:
                    codePage = codePageList[5];
                    break;

                case PrinterManager.CODE_PAGE_865:
                    codePage = codePageList[6];
                    break;

                case PrinterManager.CODE_PAGE_857:
                    codePage = codePageList[7];
                    break;

                case PrinterManager.CODE_PAGE_737:
                    codePage = codePageList[8];
                    break;

                case PrinterManager.CODE_PAGE_866:
                    codePage = codePageList[9];
                    break;

                case PrinterManager.CODE_PAGE_852:
                    codePage = codePageList[10];
                    break;

                case PrinterManager.CODE_PAGE_858:
                    codePage = codePageList[11];
                    break;

                case PrinterManager.CODE_PAGE_855:
                    codePage = codePageList[12];
                    break;

                case PrinterManager.CODE_PAGE_864:
                    codePage = codePageList[13];
                    break;

                case PrinterManager.CODE_PAGE_1250:
                    codePage = codePageList[14];
                    break;

                case PrinterManager.CODE_PAGE_1251:
                    codePage = codePageList[15];
                    break;

                case PrinterManager.CODE_PAGE_1253:
                    codePage = codePageList[16];
                    break;

                case PrinterManager.CODE_PAGE_1254:
                    codePage = codePageList[17];
                    break;

                default:
                    codePage = "";
                    break;
            }

            return codePage;
        }
    }


}